/* 
	18f4520_flash.h
	
	Flash Writes and Erases
	
	Jonathan Karpick
	8 May 2006
*/

#include "18f4520_flash.h"

// writes byteNum bytes from RAM (held in buff) to FLASH ROM starting at address
// byteNum should be <= 256 ??
void WriteFlash( near unsigned short long address, unsigned char *buff, unsigned int byteNum )
{
	unsigned int count=0;

	INTCONbits.GIE = 0;			// disable interrupts
	
	if (byteNum & (WRITE_LENGTH-1))		// if byteNum is not a multiple of WRITE_LENGTH, add WRITE_LENGTH
		byteNum += WRITE_LENGTH;	
	
	byteNum = byteNum / WRITE_LENGTH;
	
	
	do {
		memcpyram2pgm(address+count, buff+count, WRITE_LENGTH);
		TBLPTR--;					// get table pointer back into the data set
		EECON1bits.EEPGD = 1;		// point to FLASH program memory
		EECON1bits.CFGS = 0;		// access FLASH program memory
		EECON1bits.WREN = 1;		// enable write to memory
		EECON2 = 0x55;
		EECON2 = 0xAA;
		EECON1bits.WR = 1;			// start program (CPU stall)
		Nop();
		EECON1bits.WREN = 0;		// disable write to memory
		count += WRITE_LENGTH;
	} while (--byteNum);
	
	INTCONbits.GIE = 1;					// re-enable interrupts

}	
	


//---------------------------------------------------------------//
//								
//	ERASE_FLASH					
//			Erase 64 bytes from flash					
//---------------------------------------------------------------//
//	NOTE FOR ERASE_FLASH:					
//	-> BYTE_NUM USED FOR CONSECUTIVE BLOCK ERASING!		
// 	-> GIE IS SET BY ROUTINE				
//---------------------------------------------------------------//
void EraseFlash( near unsigned short long address, unsigned int byteNum )
{
	near unsigned short long tempTBLPTR;

	INTCONbits.GIE = 0;		// disable interrupts

	tempTBLPTR = TBLPTR;
	
	TBLPTR = address;
	
	if( byteNum % ERASE_LENGTH)
		byteNum = (byteNum / ERASE_LENGTH) + 1;
	else
		byteNum = (byteNum / ERASE_LENGTH);

	if( address + (unsigned short long)(byteNum*ERASE_LENGTH) > FLASH_END)
		byteNum = (unsigned int)((FLASH_END-address)/ERASE_LENGTH);

	while ( byteNum ) {
		EECON1bits.EEPGD = 1;	// point to FLASH program memory
		EECON1bits.CFGS = 0;	// access FLASH program memory
		EECON1bits.WREN = 1;	// enable write to memory
		EECON1bits.FREE = 1;	// enable row erase operation
		EECON2 = 0x55;			// write 55h
		EECON2 = 0xAA;			// write AAh
		EECON1bits.WR = 1;		// start erase (CPU stall)
		Nop();
		TBLPTR = TBLPTR + (near unsigned short long) ERASE_LENGTH;
		byteNum--;
	}
	TBLPTR = tempTBLPTR;
	INTCONbits.GIE = 1;
}


/*******************/
// ReadFlash
//    Reads byteNum bytes of characters from address in FLASH to buff in RAM
void ReadFlash( near unsigned short long address, unsigned char *buff, unsigned int byteNum )
{
	unsigned int count=0;
	unsigned char bytes;

	do {
		if( byteNum > READ_LENGTH)
			bytes=READ_LENGTH;
		else
			bytes = (unsigned char) byteNum;
		memcpypgm2ram( buff+count, (rom far char*) (address+count), bytes);
		EECON1bits.EEPGD = 1;		// point to FLASH program memory
		EECON1bits.CFGS = 0;		// access FLASH program memory
		EECON1bits.WREN = 1;		// enable write to memory
		EECON2 = 0x55;
		EECON2 = 0xAA;
		EECON1bits.WR = 1;			// start program (CPU stall)
		Nop();
		EECON1bits.WREN = 0;		// disable write to memory
		count += bytes;
		byteNum -= bytes;
	} while (byteNum);
}


