// I2C_HW.c
// Implements I2C Master Mode using the MSSP
//
//  Jonathan Karpick - 06 February 2006 (Assembly Version)
//					 - 08 May 2006 (C Version)
//  This is where you define all of the slave devices' addresses

#include "I2C_HW.h"


/************module variables*****************/
static unsigned char mDatai;
static unsigned char mDatao;
static unsigned char mTryCount;
static unsigned char mAddresses[NUM_SENSORS];

	union bits {
  struct {
    unsigned succ:1;
    unsigned bit1:1;
    unsigned bit2:1;
    unsigned bit3:1;
    unsigned bit4:1;
    unsigned bit5:1;
    unsigned bit6:1;
    unsigned bit7:1;
  };
  unsigned char byte;
};
static union bits mFlags;

/*******************Initialize Sensor Data*************/
void InitSensors( void )
{
	char i;
	
	i=NUM_SENSORS-1;
	do {
		gSensorValue[i]=0;
	} while (--i);
	
	#if (NUM_SENSORS>0)
		mAddresses[0]=ADDRESS0;
	#endif
	#if (NUM_SENSORS>1)
		mAddresses[1]=ADDRESS1;
	#endif
	#if (NUM_SENSORS>2)
		mAddresses[2]=ADDRESS2;
	#endif
	#if (NUM_SENSORS>3)
		mAddresses[3]=ADDRESS3;
	#endif
	#if (NUM_SENSORS>4)
		mAddresses[4]=ADDRESS4;
	#endif
	#if (NUM_SENSORS>5)
		mAddresses[5]=ADDRESS5;
	#endif
	#if (NUM_SENSORS>6)
		mAddresses[6]=ADDRESS6;
	#endif
	#if (NUM_SENSORS>7)
		mAddresses[7]=ADDRESS7;
	#endif
	#if (NUM_SENSORS>8)
		mAddresses[8]=ADDRESS8;
	#endif
	#if (NUM_SENSORS>9)
		mAddresses[9]=ADDRESS9;
	#endif
	//etc...

}


/*****************Initialize I2C Operations**********/
void InitI2C( void )
{
	//OpenI2C(MASTER, SLEW_OFF);  Sets SMBus Select bit??
	
	TRISCbits.TRISC3 = 1;	//SCL
	TRISCbits.TRISC4 = 1;	//SDA
	SSPSTAT = 0b10000000;
	SSPADD = I2CNUMBR;
	SSPCON1 = 0b00101000;
	SSPCON2 = 0;
	PIR1bits.SSPIF = 0;
	PIR2bits.BCLIF = 0;
		
	// check:
	// TRISC,SCL(3)=1; TRISC,SDA(4)=1; 
	// SSPSTAT=0b10000000; (SMP=1)
	// SSPADD = I2CNUMBR;
	// SSPCON1=0b00101000;
	// SSPCON2 = 0;
	// PIR1,SSPIF=0;
	// PIR2,BCLIF=0;
	
	mDatai = 0;
	mDatao = 0;
	mTryCount = 0;
	mFlags.byte = 0;
}


	

//*******************Start bit subroutine**************************
//           This routine generates a Start condition
//           (high-to-low transition of SDA while SCL
//           is still high.
//*****************************************************************
void BStart( void )
{
	mTryCount++;
	PIR1bits.SSPIF=0;
	PIR2bits.BCLIF=0;
	SSPCON2bits.SEN=1;
	while( !PIR1bits.SSPIF ) {
		if (PIR2bits.BCLIF)
			BStart();
	}
	
	//StartI2C();
	//IdleI2C();

}
	
	
/*	incf	trycount,F			// increment the try counter
    bcf     PIR1,SSPIF          // Clear SSP interrupt flag
	bcf		PIR2,BCLIF			//
    bsf     SSPCON2,SEN         // Generate Start condition
bstart_wait
    btfsc   PIR1,SSPIF          // Check if operation completed
	goto	bstart_done

	btfsc	PIR2,BCLIF			// Check for bus collision
	goto 	BSTART				// restart

    goto    bstart_wait         // If not, keep checking

bstart_done
    return
*/

//*******************Restart bit subroutine**************************
//           This routine generates a Repeated Start
//           condition (high-to-low transition of SDA
//           while SCL is still high.
//*****************************************************************
void BRestart( void )
{
	PIR1bits.SSPIF=0;
	RestartI2C();
	IdleI2C();
}
/*
BRESTART
    bcf     PIR1,SSPIF          ; Clear SSP interrupt flag
    bsf     SSPCON2,RSEN        ; Generate Restart condition
brestart_wait
    btfss   PIR1,SSPIF          ; Check if operation completed
    goto    brestart_wait       ; If not, keep checking

    return
*/

//*******************Stop bit subroutine***************************
//           This routine generates a Stop condition
//           (low-to-high transition of SDA while SCL
//           is still high.
//*****************************************************************
void BStop( void )
{
	PIR1bits.SSPIF=0;
	SSPCON2bits.PEN = 1;		// Generate STOP
	while( !PIR1bits.SSPIF);	// Wait for operation to complete
}
/*
	StopI2C();
	IdleI2C();

BSTOP
    bcf     PIR1,SSPIF          ; Clear SSP interrupt flag
    bsf     SSPCON2,PEN         ; Generate Stop condition
bstop_wait
    btfss   PIR1,SSPIF          ; Check if operation completed
    goto    bstop_wait          ; If not, keep checking

    return
*/     

//*******************Data transmit subroutine**********************
//           This routine transmits the byte of data
//           stored in 'mDatao' to the serial bus.
//*****************************************************************
void TXByte( void )
{
	mFlags.succ = 1;
	PIR1bits.SSPIF = 0;
	PIR2bits.BCLIF = 0;
	SSPCON2bits.ACKSTAT = 0;
	SSPBUF = mDatao;
	while( !PIR1bits.SSPIF ) {
		if( PIR2bits.BCLIF) {
			mFlags.succ = 0;
			return;
		}
		if (SSPCON1bits.WCOL) {
			mFlags.succ = 0;
			return;
		}
	}
	if( SSPCON2bits.ACKSTAT )	// == NACK received
		mFlags.succ = 0;
}
	
	
		
	
	
	
/*	if (WriteI2C(mDatao))
		mFlags.succ = 0;
	
	if (SSPCON2bits.ACKSTAT)
		mFlags.succ = 0;
*/		

/*		
TX_BYTE
	bsf		succflag
    bcf     PIR1,SSPIF          // Clear SSP interrupt flag
	bcf		PIR2,BCLIF			// Clear Bus collision flag
	bcf		SSPCON2,ACKSTAT		// Clear ACK flag
    movf    datao,W             // Copy datao to WREG
    movwf   SSPBUF              // Write byte out to device
tx_wait
	btfsc	PIR2,BCLIF			// Check for Bus collision
	goto 	TX_FAIL
	btfsc	SSPCON1,WCOL		// Check for Write Collision
	goto	TX_FAIL
    btfss   PIR1,SSPIF          // Check if operation completed
    goto    tx_wait             // If not, keep checking
    btfss   SSPCON2,ACKSTAT     // Check if ACK bit was received
	goto	TX_SUCCESS			// ACK_RECEIVED
TX_FAIL
	bcf		succflag
TX_SUCCESS
    return
*/

//*******************Data receive subroutine***********************
//           This routine reads in one byte of data from
//           the serial bus, and stores it in
//           mDatai.  It then responds with either an
//           ACK or a NO ACK bit, depending on the value
//           of 'ACKDT' in 'SSPCON2'.
//*****************************************************************
void RXByte( void )
{
	PIR1bits.SSPIF = 0;			// Clear SSP interrupt flag
	SSPCON2bits.RCEN = 1;		// Initiate reception of byte
	while( !PIR1bits.SSPIF);	// Wait for operation to complete
	mDatai = SSPBUF;
	PIR1bits.SSPIF = 0;
	SSPCON2bits.ACKDT = 0;		// Set up to send ACK
	SSPCON2bits.ACKEN = 1;		// Send ACK
	while( !PIR1bits.SSPIF);	// Wait for operation to complete
}	

/*
	mDatai = ReadI2C();
	AckI2C();


RX_BYTE
    bcf     PIR1,SSPIF          // Clear SSP interrupt flag
    bsf     SSPCON2,RCEN        // Initiate reception of byte
rx_wait
    btfss   PIR1,SSPIF          // Check if operation completed
    goto    rx_wait             // If not, keep checking
    movf    SSPBUF,W            // Copy byte to WREG
    movwf   datai               // Copy WREG to datai
    bcf     PIR1,SSPIF          // Clear SSP interrupt flag
    bsf     SSPCON2,ACKEN       // Generate ACK/NO ACK bit    
rx_wait2
    btfss   PIR1,SSPIF          // Check if operation completed
    goto    rx_wait2            // If not, keep checking
    return


ackfailed
	call BSTOP
	call BSTART
	call TX_BYTE
	return
*/	

/**** Read values form sensors and do simple control */
void UpdateSensors( void )
{
	unsigned char i = NUM_SENSORS-1;
	
	G_LED = 1;			// Start Update Sensors
	
	do {
		BStart();
		mDatao = mAddresses[i];
		TXByte();
		if (mFlags.succ) {
			RXByte();
			gSensorValue[i]=mDatai;
		}
		BStop();
	} while (i--);
	
	G_LED = 0;			// End Update Sensors
}
/*
UpdateSensors macro
	local CurrSens = 0	// variable to hold current sensor number
	while CurrSens < NUMDEVICES	//loop over #sensors
	//	clrf SENSOR_#v(CurrSens)
		bcf SSPCON2,ACKSTAT
		bcf SSPSTAT,S

		// send Start
		call BSTART
		
		// Send Address Byte
		movlw ADDRESS#v(CurrSens)	// address for each sensor
		movwf datao
		call TX_BYTE

		btfss succflag		// if success, keep going, else try next sensor
		goto $+0xE			// CAREFUL - DOUBLE CHECK THIS!!  Should go to call BSTOP line
		
		// Set to send ACK
		bcf SSPCON2,ACKDT

		call RX_BYTE	// Read data byte from device
		movf datai,W
		movwf SENSOR_#v(CurrSens)

		call BSTOP		// Generate STOP

		//btfsc succflag
		
		// DO SOMETHING WITH SENSOR DATA HERE (like modify VALUE_SERV_#v(CurrSens) )
		// Quick and REALLY DIRTY addition.  NOT DOING ANYTHING WITH OVERFLOW
		movf SENSOR_#v(CurrSens),W
		addwf VALUE_SERV_#v(CurrSens),W
		movwf VALUE_CMD_#v(CurrSens)
		CurrSens += 1
	endw
	while CurrSens < NUM_OF_SERVO_CHANN
		movf VALUE_SERV_#v(CurrSens),W
		movwf VALUE_CMD_#v(CurrSens)
		CurrSens += 1
	endw
	endm
*/

