/* 
   Keys18F.c

	Key management, specifically for "PIC18FXXX"
	Based on assembly code by Virgilio Mattoli
	v1.3	28 Oct 2003

	Translated to C
	Jonathan Karpick
	8 May 2006
*/

//****************************************************************************
//----------------------------------------------------------------------------
//	Keys must have pull ups and be normally high. 
//----------------------------------------------------------------------------
//	The keys must be defined in the header file.
//	When a key is pressed, the appropriate routine is called
//
//	Usage: Insert WaitForKey() in the main lop (polling)
//
//	INOTRE DEVONO ESSERE DEFINITE LE SUBROUTINE "KEY_N_Routine"
//	CHE VENGONO CHIAMATE ALLA PRESSIONE DEL TASTO "KEY_n"...
//	...E LE SUBROUTINE "KEY_N_POST_Routine"
//	CHE VENGONO CHIAMATE AL RILASCIO DEL TASTO "KEY_n".
//
//****************************************************************************

#include "Keys18F.h"

/****** module variables **************/
union bits {
  struct {
    unsigned bit0:1;
    unsigned bit1:1;
    unsigned bit2:1;
    unsigned bit3:1;
    unsigned bit4:1;
    unsigned bit5:1;
    unsigned bit6:1;
    unsigned bit7:1;
  };
  unsigned char byte;
};
static union bits mKey;

//****************************************************************************
//****************************************************************************
//
//	Key Initialization - Has some port-specific stuff in it...
//
//****************************************************************************
//****************************************************************************
void InitKeys( void )
{
	#ifdef KEY_0
		PORT_KEY_TRISbits.TRISE0 = 1;
	#endif
	#ifdef KEY_1
		PORT_KEY_TRISbits.TRISE1 = 1;
	#endif
	#ifdef KEY_2
		PORT_KEY_TRISbits.TRISE2 = 1;
	#endif
	#ifdef KEY_3
		PORT_KEY_TRISbits.TRISE3 = 1;
	#endif
	
	mKey.byte = 0;
}


//****************************************************************************
//****************************************************************************
//
//	Principle polling routing
//     Greatly simplified from original assembly - only performs debouncing
//****************************************************************************
//****************************************************************************
void WaitForKey( void )
{
	mKey.byte = PORT_KEY & MASK_KEY;
	
	if (mKey.byte) {
		KeyDecode();
		DeBounce();
	}
}


/*************************************************************************/
//	Routine to decode the keys
/*************************************************************************/
void KeyDecode( void )
{
	#ifdef KEY_0
		if (mKey.bit0)
			Key0Routine();
	#endif
	#ifdef KEY_1
		if (mKey.bit1)
			Key1Routine();
	#endif
	// etc.
}

/************************************************/
// Debouncing routine
void DeBounce( void )
{
//	R_LED = 1;		// Key Debounce Timing Start
	MILLISECONDS (DEBOUNCE_MS);
//	R_LED = 0;		// Key Debounce Timing End
}

