/* 
   Keys18F.h

	Key management, specifically for "PIC18FXXX"
	Based on assembly code by Virgilio Mattoli
	v1.3	28 Oct 2003

	Translated to C
	Jonathan Karpick
	8 May 2006
*/

//****************************************************************************
//----------------------------------------------------------------------------
//	Keys must have pull ups and be normally high. 
//----------------------------------------------------------------------------
//	The keys must be defined in the header file.
//	When a key is pressed, the appropriate routine is called
//
//	Usage: Insert WaitForKey() in the main lop (polling)
//
//	INOTRE DEVONO ESSERE DEFINITE LE SUBROUTINE "KEY_N_Routine"
//	CHE VENGONO CHIAMATE ALLA PRESSIONE DEL TASTO "KEY_n"...
//	...E LE SUBROUTINE "KEY_N_POST_Routine"
//	CHE VENGONO CHIAMATE AL RILASCIO DEL TASTO "KEY_n".
//
//****************************************************************************

#ifndef __KEYS18F
#define __KEYS18F

#include "ServoConfig.h"
#include "18fmacro.h"

//-----	HARDWARE SETTING  -------------------------------------

#define		KEY_0	0	// Corresponds to the bit of the chosen port
#define 	KEY_1	1	//
/*#undef 		KEY_2	2	//
#undef 		KEY_3	3	//
#undef 		KEY_4	4	//
#undef 		KEY_5	5	//
#undef 		KEY_6	6	//
#undef	 	KEY_7	7	//
*/

#define PORT_KEY			PORTE		// Port used for the keys
#define PORT_KEYbits 		PORTEbits
#define PORT_KEY_TRIS		TRISE	//
#define PORT_KEY_TRISbits 	TRISEbits

//-----	SOFTWARE SETTING  -------------------------------------

#undef KEYS_RETURN		// ABILITAZIONE RILASCIO TASTO
#define SPEED_KEYS_ON		// Fast Keys Qualification??
#define KEY_PRESSING		// Held key pressed ??
#undef SPEED_KEYS_RETURN	// ABILITAZIONE RILASCIO TASTO DA TASTI VELOCI

// NOTA:	If "18fmacro" is included, all the times are real
//	otherwise, the times refer to 4MPS
//
#define DEBOUNCE_MS	200	// (ms) for anti-bounce
#define WAIT_SPEED	20	// (ms * DEBOUNCE_MS) for key repetition
#define SPEED		50	// (ms) time of key repetition


#define MASK_KEY 0b00001111		// Which bits on PORT_KEY are active

/****** function prototypes **************/
void InitKeys( void );
void WaitForKey( void );
void KeyDecode( void );
void DeBounce( void );

#endif
