/*
	ServoConfig.h
	
	Header file containing setup for servo controller
	
	Jonathan Karpick
	7 May 2006
	
*/

#ifndef __SERVOCONFIG
#define __SERVOCONFIG

#include <p18cxxx.h>	//MPLAB converts this to the correct processor header

#define FREQUENCY 40000

#define	MAX_NUM_OF_SEQ 16
#define	NUM_OF_SERVO_CHAN	12  // NUM_OF_SERVO_CHANN IS DIFFERENT TO NUM_OF_SERVO ????
#define NUM_SENSORS	4

#define	G_LED			(PORTCbits.RC0)
#define LED_GREEN_TRIS	(TRISCbits.TRISC0)
#define	R_LED			(PORTCbits.RC1)
#define LED_RED_TRIS 	(TRISCbits.TRISC1)

#define	FLASH_INDEX_OFFSET	0x2100
#define FLASH_OFFSET		0x2180
//#define	FLASH_OFFSET		(FLASH_INDEX_OFFSET+0x0080)


#define FLASH_END	0x7DBF
//#define GAIT_SIZE	(FLASH_END-FLASH_INDEX_OFFSET)/64
//#define GAIT_SIZE	0x172
#define GAIT_SIZE	0x5CBF

#define	SEQ_IN_0	PORTAbits.RA0
#define	SEQ_TRIS_0	TRISAbits.TRISA0
#define	SEQ_IN_1	PORTAbits.RA1
#define	SEQ_TRIS_1	TRISAbits.TRISA1
#define	SEQ_IN_2	PORTAbits.RA2
#define	SEQ_TRIS_2	TRISAbits.TRISA2
#define	SEQ_IN_3	PORTAbits.RA3
#define	SEQ_TRIS_3	TRISAbits.TRISA3
#define SEQ_IN		PORTA
#define SEQ_MASK	0x0F

#define TOC_SIZE ((MAX_NUM_OF_SEQ*4)+2)

#define NUM_OF_SERV_P1	8		// I SERVI DEVONO ESSERE MESSI SU PIN CONSECUTIVI A PARTIRE DA 0. 
#define PORT_SERV_P1	PORTD	// DEFINIZIONE PORTA PER I SERVI
#define TRIS_SERV_P1	TRISD	//


#define NUM_OF_SERV_P2	4		// I SERVI DEVONO ESSERE MESSI SU PIN CONSECUTIVI A PARTIRE DA 0
#define PORT_SERV_P2	PORTB	// DEFINIZIONE PORTA PER I SERVI
#define TRIS_SERV_P2	TRISB	//

#define NUM_OF_SERV_P3	0		// I SERVI DEVONO ESSERE MESSI SU PIN CONSECUTIVI A PARTIRE DA 0
#define PORT_SERV_P3	PORTC	// DEFINIZIONE PORTA PER I SERVI
#define TRIS_SERV_P3	TRISC	//

#define NUM_OF_SERV	(NUM_OF_SERV_P1 + NUM_OF_SERV_P2 + NUM_OF_SERV_P3)	// TOT NUM OF SERVOS



#define STATUS_SEQ_NUM1 = 0x01
#define STATUS_SEQ_NUM2 = 0x02
#define STATUS_SEQ_NUM3 = 0x04
#define STATUS_SEQ_NUM4 = 0x08

/*#define DELAYMS(millisecs) ((FREQUENCY/4)*millisecs/10 < 255 ? Delay10TCYx(FREQUENCY*millisecs/40); : \
		(FREQUENCY/4)*millisecs/100 < 255 ? Delay100TCYx(FREQUENCY*millisecs/400); : \
		(FREQUENCY/4)*millisecs/1000 < 255 ? Delay1KTCYx(FREQUENCY*millisecs/4000); : \
 		(FREQUENCY/4)*millisecs/10000 < 255 ? Delay10KTCYx(FREQUENCY*millisecs/40000); : \
		#error Delay Too Long ) 	//converts milliseconds to instruction cycles
*/



// Global Variables
#pragma udata access globalData
extern near unsigned char gValueServPrev[NUM_OF_SERVO_CHAN];
extern near unsigned char gValueServNext[NUM_OF_SERVO_CHAN];
extern near unsigned char gValueServ[NUM_OF_SERVO_CHAN];
extern near unsigned char gValueCmd[NUM_OF_SERVO_CHAN];
extern near unsigned char gSensorValue[NUM_SENSORS];
extern near unsigned char gMov, gEmuStatusSeqNum;
extern near char gEmuMode;
extern near char gStatusSeqNum;
extern near unsigned int gStepCounter;
extern near unsigned int gTimeStep;
extern near unsigned int gTimeStepNext;
extern near unsigned int gTimeStepPrev;
extern near unsigned char gFiftieths, gQuarters, gSeconds, gMinutes;
extern near unsigned char gFrame; 
extern near unsigned short long gGaitPosn;	
extern near unsigned int  gMaxStep;	


#pragma udata buffSec
extern unsigned char gRamBuff[64];		// RAM buffer for FLASH writes

#pragma udata buffToc
extern unsigned char gTOC[TOC_SIZE];		// RAM TOC

#pragma udata

#include "18fmacro.h"

/***********************
 * Global Functions
 ***************************/
//void SetSeqStartParam( void );
void LoadTOC( void );
void SaveTOC( void );
void Key0Routine( void );
void Key0PressRoutine( void );
void Key1Routine( void );
void Key1PressRoutine( void );


#endif 
