/*
	ServoF18Extnd.c

	Servo Routines for Microchip 18F series PICs and HS-81 servos.

	Original Assembly by Virgilio Mattoli et al. v2.1 15 July 2004
	Converted to C by Jonathan Karpick
	8 May 2006
*/

//********************************************************************************************
//
//	NOTE: To function, "ServoUpdate" must be called every 20ms!
//		  (interrupt on CCP Timer #1).
//		  f18macro.asm must be included... (to be changed??)
//				
//********************************************************************************************


#include "ServoF18Extnd.h"



// Module Variables
union bits {
  struct {
    unsigned bit0:1;
    unsigned bit1:1;
    unsigned bit2:1;
    unsigned bit3:1;
    unsigned bit4:1;
    unsigned bit5:1;
    unsigned bit6:1;
    unsigned bit7:1;
  };
  unsigned char byte;
};
static union bits mServTempP1;
static union bits mServTempP2;
static union bits mServTempP3;
/*
static unsigned char mServTempP1;
static unsigned char mServTempP2;
static unsigned char mServTempP3;
*/

/****** Initialize Servo Output Pins  ****************
 * In default setup (12 servos), ports should be:
 *  PORTD = 0
 *  TRISD = 0
 *  PORTB = 0x?0
 *  TRISB = 0x?0
*/
void ServoInit( void )
{
#if (NUM_OF_SERV_P1 > 0)
	TRIS_SERV_P1 = (~MASK_SERV_P1)&TRIS_SERV_P1;
	PORT_SERV_P1 = (~MASK_SERV_P1)&PORT_SERV_P1;
#endif

#if (NUM_OF_SERV_P2 > 0)
	TRIS_SERV_P2 = (~MASK_SERV_P2)&TRIS_SERV_P2;
	PORT_SERV_P2 = (~MASK_SERV_P2)&PORT_SERV_P2;
#endif

#if (NUM_OF_SERV_P3 > 0)
	TRIS_SERV_P3 = (~MASK_SERV_P3)&TRIS_SERV_P3;
	PORT_SERV_P3 = (~MASK_SERV_P3)&PORT_SERV_P3;
#endif
}

//-----------------------------------------------
//	(2 * NUM_OF_SERV) + 5 instructions	= 29 (including call and return)
void N_Servi( void )
{	_asm 
	movf gFrame,0,1 
	#if (NUM_OF_SERV_P1 > 0) 
		CPFSGT	gValueCmd,1 
		BCF	mServTempP1,0,1 
	#endif 
	#if (NUM_OF_SERV_P1 > 1) 
		CPFSGT	gValueCmd+1,1 
		BCF	mServTempP1,1,1
	#endif
	#if (NUM_OF_SERV_P1 > 2)
		CPFSGT	gValueCmd+2,1
		BCF	mServTempP1,2,1
	#endif
	#if (NUM_OF_SERV_P1 > 3)
		CPFSGT	gValueCmd+3,1
		BCF	mServTempP1,3,1
	#endif
	#if (NUM_OF_SERV_P1 > 4)
		CPFSGT	gValueCmd+4,1
		BCF	mServTempP1,4,1
	#endif
	#if (NUM_OF_SERV_P1 > 5) 
		CPFSGT	gValueCmd+5,1
		BCF	mServTempP1,5,1
	#endif
	#if (NUM_OF_SERV_P1 > 6)
		CPFSGT	gValueCmd+6,1
		BCF	mServTempP1,6,1
	#endif
	#if (NUM_OF_SERV_P1 > 7)
		CPFSGT	gValueCmd+7,1
		BCF	mServTempP1,7,1
	#endif
	#if (NUM_OF_SERV_P2 > 0) 
		CPFSGT	gValueCmd+8,1
		BCF	mServTempP2,0,1
	#endif
	#if (NUM_OF_SERV_P2 > 1) 
		CPFSGT	gValueCmd+9,1
		BCF	mServTempP2,1,1
	#endif
	#if (NUM_OF_SERV_P2 > 2) 
		CPFSGT	gValueCmd+10,1
		BCF	mServTempP2,2,1
	#endif
	#if (NUM_OF_SERV_P2 > 3) 
		CPFSGT	gValueCmd+11,1
		BCF	mServTempP2,3,1
	#endif
	#if (NUM_OF_SERV_P2 > 4)
		CPFSGT	gValueCmd+12,1
		BCF	mServTempP2,4,1
	#endif
	#if (NUM_OF_SERV_P2 > 5)
		CPFSGT	gValueCmd+13,1
		BCF	mServTempP2,5,1
	#endif
	#if (NUM_OF_SERV_P2 > 6)
		CPFSGT	gValueCmd+14,1
		BCF	mServTempP2,6,1
	#endif
	#if (NUM_OF_SERV_P2 > 7)
		CPFSGT	gValueCmd+15,1
		BCF	mServTempP2,7,1
	#endif
	#if (NUM_OF_SERV_P3 > 0)
		CPFSGT	gValueCmd+16,1
		BCF	mServTempP3,0,1
	#endif
	#if (NUM_OF_SERV_P3 > 1)
		CPFSGT	gValueCmd+17,1
		BCF	mServTempP3,1,1
	#endif
	#if (NUM_OF_SERV_P3 > 2) 
		CPFSGT	gValueCmd+18,1
		BCF	mServTempP3,2,1
	#endif 
	#if (NUM_OF_SERV_P3 > 3)
		CPFSGT	gValueCmd+19,1 
		BCF	mServTempP3,3,1 
	#endif
	#if (NUM_OF_SERV_P3 > 4)
		CPFSGT	gValueCmd+20,1
		BCF	mServTempP3,4,1
	#endif
	#if (NUM_OF_SERV_P3 > 5) 
		CPFSGT	gValueCmd+21,1 
		BCF	mServTempP3,5,1
	#endif
	#if (NUM_OF_SERV_P3 > 6)  
		CPFSGT	gValueCmd+22,1 
		BCF	mServTempP3,6,1
	#endif 
	#if (NUM_OF_SERV_P3 > 7)
		CPFSGT	gValueCmd+23,1
		BCF	mServTempP3,7,1
	#endif
	_endasm
}


//---------------------------------------------------------------------------------------//

void ServoUpdate( void )
{
	gFrame = 0;

#if (NUM_OF_SERV_P1 > 0)
	mServTempP1.byte = 0xFF;
//	mServTempP1 = 0xFF;
	PORT_SERV_P1 = PORT_SERV_P1 | MASK_SERV_P1;
#endif

#if (NUM_OF_SERV_P2 > 0)
	mServTempP2.byte = 0xFF;
//	mServTempP2 = 0xFF;
	PORT_SERV_P2 = PORT_SERV_P2 | MASK_SERV_P2;
#endif

#if (NUM_OF_SERV_P3 > 0)
	mServTempP3.byte = 0xFF;
//	mServTempP3 = 0xFF;
	PORT_SERV_P3 = PORT_SERV_P3 | MASK_SERV_P3;
#endif
		
// DelayMicroSec	D'860'	// 0.860 ms DI ATTESA (per utilizzare tutti i bit ... 
				// ... vado leggermente fuori le specifiche del servo)
	Delay100TCYx(80);	// 800 us
	Delay10TCYx(60);	// 60 us
			
	do {  // ServoLoop should run in exactly 5 us!!
	// IL LOOP, ESCLUSI I NOP, E' DI (2 * NUM_OF_SERV + 2 * NUM_SERVOPORT + 4) ISTRUZIONI <<--VERIFY!!
	
	N_Servi();			// La macro occupa (2 * NUM_OF_SERV) Istruzioni.
	//N_Nop USE A HW TIMER?? - Sim shows exactly 5us.	
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();	// Determined via simulation
	
			
	#if (NUM_OF_SERV_P1 > 0)
		PORT_SERV_P1 = PORT_SERV_P1 & mServTempP1.byte;
	#endif
	#if (NUM_OF_SERV_P2 > 0)
		PORT_SERV_P2 = PORT_SERV_P2 & mServTempP2.byte;
	#endif
	#if (NUM_OF_SERV_P3 > 0)
		PORT_SERV_P3 = PORT_SERV_P3 & mServTempP3.byte;
	#endif
	gFrame++;
	} while (gFrame != 0);
}
