/*
	ServoF18Extend.h

	Servo Routines for Microchip 18F series PICs and HS-81 servos.

	Original Assembly by Virgilio Mattoli et al. v2.1 15 July 2004
	Converted to C by Jonathan Karpick
	8 May 2006
*/

//********************************************************************************************
//
//	NOTE: To function, "ServoUpdate" must be called every 20ms!
//		  (interrupt on CCP Timer #1).
//		  f18macro.asm must be included... (to be changed??)
//				
//********************************************************************************************

//--- Registries defined in main program (to be changed...)  -----------------------------------//
//	FRAME, SERV_TEMP_P1, SERV_TEMP_P2, SERV_TEMP_P3				//
//	VALUE_SERV_P1_0, VALUE_SERV_P1_1, VALUE_SERV_P1_2, VALUE_SERV_P1_3	//
//	VALUE_SERV_P1_4, VALUE_SERV_P1_5, VALUE_SERV_P1_6, VALUE_SERV_P1_7	//
//	VALUE_SERV_P2_0, VALUE_SERV_P2_1, VALUE_SERV_P2_2, VALUE_SERV_P2_3	//
//	VALUE_SERV_P2_4, VALUE_SERV_P2_5, VALUE_SERV_P2_6, VALUE_SERV_P2_7	//
//	VALUE_SERV_P3_0, VALUE_SERV_P3_1, VALUE_SERV_P3_2, VALUE_SERV_P3_3	//
//	VALUE_SERV_P3_4, VALUE_SERV_P3_5, VALUE_SERV_P3_6, VALUE_SERV_P3_7	//
//										//
//	Nota: NON SONO TUTTI NECESSARI ... DIPENDE DALLA CONFIGURAZIONE		//
//										//

//-------------------------------//
// PARAMETRI IMPOSTABILI		//
//-------------------------------//
				//

#ifndef _SERVOF18EXTEND
#define _SERVOF18EXTEND
#include "ServoConfig.h"
#include <delays.h>

#if NUM_OF_SERV == 0
#error "Selezionare almeno un servomotore!"
#endif

#if (NUM_OF_SERV_P1 > 0)
	#define NUM_SERVO_PORT_1 1
#else
	#define NUM_SERVO_PORT_1 0
#endif

#if (NUM_OF_SERV_P2 > 0)
	#define NUM_SERVO_PORT_2 1
#else
	#define NUM_SERVO_PORT_2 0
#endif

#if (NUM_OF_SERV_P3 > 0)
	#define NUM_SERVO_PORT_3 1
#else
	#define NUM_SERVO_PORT_3 0
#endif

#define NUM_SERVOPORT (NUM_SERVO_PORT_1 + NUM_SERVO_PORT_2 + NUM_SERVO_PORT_3)
#undef NUM_SERVO_PORT_1
#undef NUM_SERVO_PORT_2
#undef NUM_SERVO_PORT_3

#if (NUM_OF_SERV_P1 == 1)
	#define MASK_SERV_P1 0b00000001
#elif (NUM_OF_SERV_P1 == 2)
	#define MASK_SERV_P1 0b00000011
#elif (NUM_OF_SERV_P1 == 3)
	#define MASK_SERV_P1 0b00000111
#elif (NUM_OF_SERV_P1 == 4)
	#define MASK_SERV_P1 0b00001111
#elif (NUM_OF_SERV_P1 == 5)
	#define MASK_SERV_P1 0b00011111
#elif (NUM_OF_SERV_P1 == 6)
	#define MASK_SERV_P1 0b00111111
#elif (NUM_OF_SERV_P1 == 7)
	#define MASK_SERV_P1 0b01111111
#elif (NUM_OF_SERV_P1 == 8)
	#define MASK_SERV_P1 0b11111111
#elif (NUM_OF_SERV_P1 > 8)
	#error "Too many servos on first port!"
#endif

#if (NUM_OF_SERV_P2 == 1)
	#define MASK_SERV_P2 0b00000001
#elif (NUM_OF_SERV_P2 == 2)
	#define MASK_SERV_P2 0b00000011
#elif (NUM_OF_SERV_P2 == 3)
	#define MASK_SERV_P2 0b00000111
#elif (NUM_OF_SERV_P2 == 4)
	#define MASK_SERV_P2 0b00001111
#elif (NUM_OF_SERV_P2 == 5)
	#define MASK_SERV_P2 0b00011111
#elif (NUM_OF_SERV_P2 == 6)
	#define MASK_SERV_P2 0b00111111
#elif (NUM_OF_SERV_P2 == 7)
	#define MASK_SERV_P2 0b01111111
#elif (NUM_OF_SERV_P2 == 8)
	#define MASK_SERV_P2 0b11111111
#elif (NUM_OF_SERV_P2 > 8)
	#error "Too many servos on second port!"
#endif


#if (NUM_OF_SERV_P3 == 1)
	#define MASK_SERV_P3 0b00000001
#elif (NUM_OF_SERV_P3 == 2)
	#define MASK_SERV_P3 0b00000011
#elif (NUM_OF_SERV_P3 == 3)
	#define MASK_SERV_P3 0b00000111
#elif (NUM_OF_SERV_P3 == 4)
	#define MASK_SERV_P3 0b00001111
#elif (NUM_OF_SERV_P3 == 5)
	#define MASK_SERV_P3 0b00011111
#elif (NUM_OF_SERV_P3 == 6)
	#define MASK_SERV_P3 0b00111111
#elif (NUM_OF_SERV_P3 == 7)
	#define MASK_SERV_P3 0b01111111
#elif (NUM_OF_SERV_P3 == 8)
	#define MASK_SERV_P3 0b11111111
#elif (NUM_OF_SERV_P3 > 8)
	#error "Too many servos on third port!"
#endif

//---------------------------------------------------------------------------------------//
//	Il numero di istruzioni in 5 us :						//
//	D'5' * FREQUENCY / D'4000'							//
//	Dunque il numero di nop da inserire :						//
//	(D'5' * FREQUENCY / D'4000') - (D'6' + D'2' * NUM_OF_SERV)			//
//---------------------------------------------------------------------------------------//
/*#define N_Nop \
	_asm variable NomNumFix \
	variable MinInstr \
	MinInstr =(4+(2*NUM_SERVOPORT)) \
	NomNumFix=(5*FREQUENCY/4000)-(MinInstr2*NUM_OF_SERV) \
	if (5*FREQUENCY/4000)<(MinInstr+2*NUM_OF_SERV) \
		error "La frequenza  troppo bassa per pilotare tutti i servimotore!" \
	endif \			
	if((5*FREQUENCY)%4000)!=0 \
		messg "Il controllo dei servi NON sar esatto, causa frequenza sistema" \
	endif \
	local	nopnum = 0	\				
	while	nopnum < NomNumFix	\
	NOP	\
	nopnum ++ \
	endw \
	_endasm
*/

void ServoInit( void );	
void N_Servi( void );
void ServoUpdate( void );


#endif
