/* 
	ServoRoutines.c
	SetPorts(), VarInit(), StatusSeqNumRead(), and Decode()

	Assembly by Virgilio Mattoli
	Adapted to C by Jonathan Karpick
	7 May 2006
*/

#include "ServoRoutines.h"


//********************************************************************************************
void SetPorts( void )
{
	INTCON = 0;		// All interrupts disabled
	ADCON1 = 0x0F;	// Voltage ref = Vdd/Vss, all A/D configured as Digital I/O
    TRISA = 0xFF;	// All Inputs
    TRISB = 0xFF;	// All Inputs
    TRISC = 0xFF;	// All Inputs
    TRISD = 0xFF;	// All Inputs
    TRISE = 0x07;	// E0:E2 Inputs

    INTCON2bits.RBPU = 1;	// DISABLE all PORTB pull-ups 
	LED_RED_TRIS = 0;   // Set LED_RED to OUTPUT
	LED_GREEN_TRIS = 0;	// Set LED_GREEN to OUTPUT
	R_LED = 0;			// Turn off Red LED - initialization
	G_LED = 0;			// Turn off Green LED - initialization

	ServoInit();	// Sets servo pins to outputs
	
	PIE1 = 0;		// All peripheral interrupts disabled
	PIE2 = 0;		// All peripheral interrupts disabled

	DelayCounter1 = 0;	// Initialize delay counters
	DelayCounter2 = 0;

	DevSetting();		// Sets up RS232 & Timer3/CCP2
	InitKeys();
}	

// ************************************
void VarInit( void )
{
	gFiftieths = 0;
	gQuarters = 0;
	gSeconds = 0;
	gMinutes = 0;
	gMov = 0;	
	gEmuStatusSeqNum = 0;
	gEmuMode = 0;
	ResVal();
}

// ***********************************	
void ResVal( void )
{	
	unsigned char i;
	
	i=NUM_OF_SERVO_CHAN-1;

	do {
		gValueServ[i] = 0x80;
		gValueCmd[i] = 0x80;
	} while( i-- );
	
	gStepCounter = 0;

	LoadTOC();	

//----- Comment out if you do not want sequence select operable
	StatusSeqNumRead();

	SetSeqStartParam();
}
	


// ********************************************
void StatusSeqNumRead( void )
{
	gStatusSeqNum = SEQ_MASK & SEQ_IN; 

	if (gEmuMode == 1)
		gStatusSeqNum = gEmuStatusSeqNum;
}


// ***************************************************************
//
//	 RS232 SERIAL COMUNICATION COMMANDS 
//
// ***************************************************************



// ***************************************************************
//
//	SERIAL COMMANDS DECODING
//
// ***************************************************************

void Decode ( void )
{
	unsigned char temp;

//	temp = RCREG;

//	TXREG = temp;	// ECHO	(UNCOMMENT ONLY FOR COMM TESTS) <---------------
	
	temp = ReadUSART();

	switch (temp) {
		case _START:
			TimerOn();
			break;
		case _START_MOV:
			StartMov();
			break;
		case _STOP:
			TimerOff();
			break;
		case _STOP_MOV:
			StopMov();
			break;
		case _S_0:
			Servo(0);
			break;
		case _S_1:
			Servo(1);
			break;
		case _S_2:
			Servo(2);
			break;
		case _S_3:
			Servo(3);
			break;
		case _S_4:
			Servo(4);
			break;
		case _S_5:
			Servo(5);
			break;
		case _S_6:
			Servo(6);
			break;
		case _S_7:
			Servo(7);
			break;
		case _S_8:
			Servo(8);
			break;
		case _S_9:
			Servo(9);
			break;
		case _S_10:
			Servo(10);
			break;
		case _S_11:
			Servo(11);
			break;
		case _LOAD:
			LoadProg();
			break;
		case _DOWNLOAD:
			SendProg();
			break;
		case _ERASE:
			Erase();
			break;
		case _RESET:
			Reset();
			break;
	}
}

void Servo( unsigned char servoNum )
{
//	while ( PIR1bits.RCIF == 0 );	// Wait for PWM Duty
//	gValueServ[servoNum] = RCREG;

	while ( !DataRdyUSART() );		// Wait for RCIF to be set
	gValueCmd[servoNum] = ReadUSART();
}	

void StartMov( void )
{
	unsigned char temp;

	while ( !DataRdyUSART() );	// Wait for RX ready
	
	temp = ReadUSART();
	
	gEmuStatusSeqNum = temp;
	gStatusSeqNum = temp;
	
	if (gEmuMode == 0) {
		gEmuMode = 1;
		TimerMovOn();
	}
}


void StopMov( void )
{
	gStatusSeqNum = 0;
	gEmuMode = 0;
	TimerMovOff();
}

// **************************************************************
//   Programs new gait into FLASH ROM
//	   *** MUST BE PRECEEDED BY AN ERASE OF DESIRED
//         FLASH LOCATION!!!!!!!!!!!!!*******
//	In current configuration, this is handled by the
//   visual basic program...
void LoadProg( void )
{
	unsigned char i;
	unsigned int tempCounter;
	unsigned char pageCounter;
	unsigned char byteNum;
	near unsigned short long address = FLASH_OFFSET;
//	far ram int* buff;
	
	R_LED = 1;			// Save Program
	
	i = MAX_NUM_OF_SEQ*4+2-1;
	
	do {
		while( !DataRdyUSART() );	// Wait for RX ready
		gTOC[(MAX_NUM_OF_SEQ*4+2-1)-i] = ReadUSART();
	} while( i-- );
	
	Nop();
	
	WriteUSART( 0xFF );		// ACK

	tempCounter = ((unsigned int)(gTOC[0])<<4) + (unsigned int)(gTOC[1]);

	while ( tempCounter != 0 ) {
		i=0;
		pageCounter = (unsigned char) tempCounter;
		byteNum = (unsigned char) tempCounter;
		if ( tempCounter > 255 ) {
			pageCounter = 0;
			byteNum = 0;
		}
		do {
			while( !DataRdyUSART() );	// Wait for RX ready
			gRamBuff[i] = ReadUSART();
			i++;
			tempCounter--;
			pageCounter--;
		} while (pageCounter);
		WriteFlash(address, gRamBuff, byteNum);
		if( byteNum )
			address += byteNum;
		else
			address += 256;
		WriteUSART( 0xFF );		// ACK
	}
	SaveTOC();
	ResVal();
	R_LED = 0;			// End of save program
}


// **************************************************************
void SendProg( void )
{
	unsigned int tempCounter;
	rom unsigned char* rombuff;

	R_LED = 1;		// Send program
	rombuff = (rom unsigned char*) FLASH_INDEX_OFFSET;

	tempCounter = MAX_NUM_OF_SEQ*4 + 2 - 1;	// 66 bytes

	do {
		RS232PutChar( *rombuff );
		rombuff++;
	} while( tempCounter--);
		
	//DelayMilliSec 500
	//DELAYMS(500)
	
	MILLISECONDS (250);	// 250 ms
	MILLISECONDS (250);	// 250 ms
	
	rombuff = (rom unsigned char*) FLASH_OFFSET;		//SEND FLASH BYTES
	tempCounter = ((unsigned int)(gTOC[0])<<4) + (unsigned int)(gTOC[1]);

	while (tempCounter != 0) {
		RS232PutChar( *rombuff );
		rombuff++;
		tempCounter--;
	}
	TBLPTR = FLASH_OFFSET;
	ResVal();
	R_LED = 0;		// End of send program
}

// **************************************************************
void Erase( void )
{
	unsigned char i;
	
	unsigned int bytes;
	
	bytes = ((unsigned int)(gTOC[0])<<4) + (unsigned int)(gTOC[1]) + (FLASH_OFFSET - FLASH_INDEX_OFFSET);

	R_LED = 1;			// Erase gait
	EraseFlash(FLASH_INDEX_OFFSET, bytes);

	i = MAX_NUM_OF_SEQ*4 + 2 - 1;
	do {
		gTOC[i] = 0;
	} while( i-- );

	WriteUSART( 0xFF );		// ACK

//	DelayMilliSec	D'100'
//	MILLISECONDS (100);	

	R_LED = 0;			// End of Erase Gait
}

// ****************************************************************************
void RS232PutChar( unsigned char out )
{
	while( !TXSTAbits.TRMT );	// Wait for USART ready
	WriteUSART(out);
}

void ReadStepFromFlash( void )
{
	rom unsigned char *buff;
	
	memcpy( gValueServPrev, gValueServNext, NUM_OF_SERVO_CHAN );
	gTimeStepPrev = gTimeStepNext;
	gTimeStep = gTimeStepPrev;
	
	if( gStepCounter >= (gMaxStep-1) ) {
		gStepCounter = 0xFFFF;		// -1...
		gGaitPosn = FLASH_OFFSET + ((unsigned int)(gTOC[gStatusSeqNum*4+4])<<4) + (unsigned int)(gTOC[gStatusSeqNum*4+5]);
	}

	memcpypgm2ram( gValueServNext, gGaitPosn, NUM_OF_SERVO_CHAN );
	buff = (rom unsigned char*)(gGaitPosn+NUM_OF_SERVO_CHAN);
	gTimeStepNext = ((unsigned int)(*buff))<<4;
	buff++;
	gTimeStepNext += ((unsigned int)(*buff));
	gGaitPosn += NUM_OF_SERVO_CHAN + 2;
}	


// Timer Routines
void TimerOn( void )
{
	G_LED = 1;			// Timer On
	TMR3L = 0;
	TMR3H = 0;
	PIR2bits.CCP2IF = 0;
	PIE2bits.CCP2IE = 1;
	T3CONbits.TMR3ON = 1;
}

void TimerOff( void )
{
	G_LED = 0;			// Timer Off
	PIR2bits.CCP2IF = 0;
	PIE2bits.CCP2IE = 0;
	T3CONbits.TMR3ON = 0;
	PIR2bits.CCP2IF = 0;
}

void TimerMovOn( void )
{
	if( gTOC[0] | gTOC[1] ) {			// check to ensure that there is a gait!!
		TimerOn();
		SetSeqStartParam();
//		ReadStepFromFlash();	// already done in SetSeqStartParam();
		R_LED = 1;		// Timer Move On
		gMov = 0xFF;
	}
}

void TimerMovOff( void )
{
	TimerOff();
	R_LED = 0;		// Timer Move Off
	gMov = 0;
}

void SetSeqStartParam( void ) 
//SET SEQUENCE PARAMETER ON THE BASE OF STATUS_SEQ_NUM
{
	ram unsigned char *ptr;
	rom unsigned char *buff;

	gStepCounter = 0;
	
	ptr = gTOC + (unsigned int)(gStatusSeqNum*4 + 2);
	gMaxStep = (((unsigned int)(*ptr))<<4) + ((unsigned int)(*(++ptr)));
	ptr++;
	gGaitPosn = FLASH_OFFSET + (((unsigned int)(*ptr))<<4) + ((unsigned int)(*(++ptr)));
	memcpypgm2ram( gValueServNext, gGaitPosn, NUM_OF_SERVO_CHAN ); // Load 1st step
	buff = (rom unsigned char*)(gGaitPosn+NUM_OF_SERVO_CHAN);
	gTimeStepNext = ((unsigned int)(*buff++))<<4;
	gTimeStepNext += ((unsigned int)(*buff++));
	gGaitPosn += NUM_OF_SERVO_CHAN+2;
	ReadStepFromFlash();
}



/********* Conduct linear interpolation between gait waypoints ********/
void Interpolate( void )
{
	unsigned char i=NUM_OF_SERVO_CHAN-1;
	long next;
	
	// comments for i=0, 1st time step of gait
	
	do {
		next = (gTimeStepPrev-gTimeStep+1)*((long)gValueServNext[i]-(long)gValueServPrev[i]);
			// 0xFFFFFFD8
		next = next / gTimeStepPrev;	// 0xFFFFFFFF
		next += gValueServPrev[i];		// 0x31 = 49	(good!)
		if( next > 255 )
			next = 255;
		if( next < 0 )
			next = 0;
		gValueServ[i] = (unsigned char) next;
	} while( i-- );
}
