/* 
	ServoRoutines.h
	SetPorts(), VarInit(), StatusSeqNumRead(), and Decode()

	Assembly by Virgilio Mattoli
	Adapted to C by Jonathan Karpick
	7 May 2006
*/

#ifndef SERVOCONFIG
#define SERVOCONFIG

#include "ServoConfig.h"
#include "ServoF18Extnd.h"
#include "18f4520_flash.h"
#include "18f_perf.h"
#include "Keys18F.h"
#include <usart.h>
#include <string.h>		// For memcpypgm2ram

extern unsigned char DelayCounter1;	// To save delay register during interrupts
extern unsigned char DelayCounter2;

#define _STOP		0b00000000		// 0x00
#define _START		0b00000001		// 0x01
#define _LOAD 	 	0b00000010		// UPDATE MEMORY
#define _START_MOV	0b00000011	
#define _STOP_MOV	0b00000100		
#define _SPEED_MOV	0b00000101	// UPDATE SPEED
#define _DOWNLOAD	0b00000110		// DOWNLOAD MEMORY
#define _ERASE	0b00000111		// ERASE MEMORY
#define _PC_EMU	0b00010000		// PC EMULATOR
#define _RESET	0b00010001		// RESET PROCESSOR

#define _S_0	0b00001000		// UPDATE SERVO 0	0x08
#define _S_1	0b00001001		// UPDATE SERVO 1	0x09
#define _S_2	0b00001010		// UPDATE SERVO 2	0x10
#define _S_3	0b00001011		// UPDATE SERVO 3	0x11
#define _S_4	0b00001100		// UPDATE SERVO 4	0x12
#define _S_5	0b00001101		// UPDATE SERVO 5	0x13
#define _S_6	0b00001110		// UPDATE SERVO 6	0x14
#define _S_7	0b00001111		// UPDATE SERVO 7	0x15
#define _S_8	0b00101000		// UPDATE SERVO 8	
#define _S_9	0b00101001		// UPDATE SERVO 9
#define _S_10	0b00101010		// UPDATE SERVO 10
#define _S_11	0b00101011		// UPDATE SERVO 11
//#define _S_12	0b00101100		// UPDATE SERVO 12
//#define _S_13	0b00101101		// UPDATE SERVO 13
//#define _S_14	0b00101110		// UPDATE SERVO 14
//#define _S_15	0b00101111		// UPDATE SERVO 15

void SetPorts( void );
void VarInit( void );
void ResVal( void );
void StatusSeqNumRead( void );
void Decode ( void );
void Servo( unsigned char servoNum );
void StartMov( void );
void StopMov( void );
void LoadProg( void );
void SendProg( void );
void Erase( void );
void RS232PutChar( unsigned char out );
void ReadStepFromFlash( void );
void TimerOn( void );
void TimerOff( void );
void TimerMovOn( void );
void TimerMovOff( void );
void SetSeqStartParam( void );
void Interpolate( void );


#endif
